@@ -1,12 +1,12 @@
 /*
    dcraw.h - Dave Coffin's raw photo decoder - header for C++ adaptation
-   Copyright 1997-2007 by Dave Coffin, dcoffin a cybercom o net
-   Copyright 2004-2007 by Udi Fuchs, udifuchs a gmail o com
+   Copyright 1997-2008 by Dave Coffin, dcoffin a cybercom o net
+   Copyright 2004-2008 by Udi Fuchs, udifuchs a gmail o com
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   as published by the Free Software Foundation. You should have received
-   a copy of the license along with this program.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
    This is a adaptation of Dave Coffin's original dcraw.c to C++.
    It can work as either a command-line tool or called by other programs.
@@ -38,7 +38,7 @@ class DCRaw { public:
 /* All dcraw's global variables are members of this class. */
 FILE *ifp;
 short order;
-char *ifname, *meta_data;
+char *ifname, *ifname_display, *meta_data;
 char cdesc[5], desc[512], make[64], model[64], model2[64], artist[64];
 float flash_used, canon_ev, iso_speed, shutter, aperture, focal_len;
 time_t timestamp;
@@ -50,7 +50,7 @@ unsigned thumb_misc, *oprof, fuji_layout, shot_select, multi_out;
 unsigned tiff_nifds, tiff_samples, tiff_bps, tiff_compress;
 unsigned black, maximum, mix_green, raw_color, use_gamma, zero_is_bad;
 unsigned zero_after_ff, is_raw, dng_version, is_foveon, data_error;
-unsigned tile_width, tile_length;
+unsigned tile_width, tile_length, gpsdata[32], load_flags;
 ushort raw_height, raw_width, height, width, top_margin, left_margin;
 ushort shrink, iheight, iwidth, fuji_width, thumb_width, thumb_height;
 int flip, tiff_flip, colors;
@@ -60,6 +60,7 @@ float bright, user_mul[4], threshold;
 int half_size, four_color_rgb, document_mode, highlight;
 int verbose, use_auto_wb, use_camera_wb, use_camera_matrix;
 int output_color, output_bps, output_tiff, med_passes;
+int no_auto_bright;
 unsigned greybox[4];
 float cam_mul[4], pre_mul[4], cmatrix[3][4], rgb_cam[3][4];
 int histogram[4][0x2000];
@@ -90,6 +91,7 @@ int lastStatus;
 
 /* Initialization of the variables is done here */
 DCRaw();
+~DCRaw();
 void dcraw_message(int code, const char *format, ...);
 /* All dcraw functions with the CLASS prefix are members of this class. */
 int fc (int row, int col);
@@ -103,6 +105,7 @@ unsigned getint (int type);
 float int_to_float (int i);
 double getreal (int type);
 void read_shorts (ushort *pixel, unsigned count);
+void canon_black (double dark[2]);
 void canon_600_fixed_wb (int temp);
 int canon_600_color (int ratio[2], int mar);
 void canon_600_auto_wb();
@@ -127,14 +130,12 @@ void adobe_dng_load_raw_lj();
 void adobe_dng_load_raw_nc();
 void pentax_k10_load_raw();
 void nikon_compressed_load_raw();
-void nikon_load_raw();
 int nikon_is_compressed();
 int nikon_e995();
 int nikon_e2100();
 void nikon_3700();
 int minolta_z2();
 void nikon_e900_load_raw();
-void nikon_e2100_load_raw();
 void fuji_load_raw();
 void jpeg_thumb (FILE *tfp);
 void ppm_thumb (FILE *tfp);
@@ -153,6 +154,8 @@ void sinar_4shot_load_raw();
 void imacon_full_load_raw();
 void packed_12_load_raw();
 void unpacked_load_raw();
+void nokia_load_raw();
+unsigned pana_bits (int nbits);
 void panasonic_load_raw();
 void olympus_e300_load_raw();
 void olympus_e410_load_raw();
@@ -167,6 +170,7 @@ void kodak_radc_load_raw();
 void kodak_jpeg_load_raw();
 void kodak_dc120_load_raw();
 void kodak_easy_load_raw();
+void kodak_yrgb_load_raw();
 void kodak_262_load_raw();
 int kodak_65000_decode (short *out, int bsize);
 void kodak_65000_load_raw();
@@ -195,7 +199,7 @@ void foveon_make_curves
 	(short **curvep, float dq[3], float div[3], float filt);
 int foveon_apply_curve (short *curve, int i);
 void foveon_interpolate();
-void bad_pixels();
+void bad_pixels(char *fname);
 void subtract(const char *fname);
 void pseudoinverse (double (*in)[3], double (*out)[3], int size);
 void cam_xyz_coeff (double cam_xyz[4][3]);
@@ -218,6 +222,7 @@ void parse_thumb_note (int base, unsigned toff, unsigned tlen);
 void parse_makernote (int base, int uptag);
 void get_timestamp (int reversed);
 void parse_exif (int base);
+void parse_gps (int base);
 void romm_coeff (float romm_cam[3][3]);
 void parse_mos (int offset);
 void linear_table (unsigned len);
