@@ -2,21 +2,22 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * dcraw_api.cc - API for DCRaw
- * Copyright 2004-2007 by Udi Fuchs
+ * Copyright 2004-2008 by Udi Fuchs
  *
  * based on dcraw by Dave Coffin
  * http://www.cybercom.net/~dcoffin/
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2
- * as published by the Free Software Foundation. You should have received
- * a copy of the license along with this program.
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <math.h> /* for sqrt() */
@@ -41,8 +42,8 @@ void wavelet_denoise_INDI(gushort (*image)[4], const int black,
 void scale_colors_INDI(gushort (*image)[4], const int maximum, const int black,
     const int use_auto_wb, const int use_camera_wb, const float cam_mul[4],
     const int iheight, const int iwidth, const int colors, float pre_mul[4],
-    const unsigned filters, /*const*/ gushort white[8][8], const char *ifname,
-    void *dcraw);
+    const unsigned filters, /*const*/ gushort white[8][8],
+    const char *ifname_display, void *dcraw);
 void lin_interpolate_INDI(gushort (*image)[4], const unsigned filters,
     const int width, const int height, const int colors, void *dcraw);
 void vng_interpolate_INDI(gushort (*image)[4], const unsigned filters,
@@ -60,15 +61,19 @@ void flip_image_INDI(gushort (*image)[4], int *height_p, int *width_p,
 void fuji_rotate_INDI(gushort (**image_p)[4], int *height_p, int *width_p,
     int *fuji_width_p, const int colors, const double step, void *dcraw);
 
-int dcraw_open(dcraw_data *h,char *filename)
+int dcraw_open(dcraw_data *h, char *filename)
 {
     DCRaw *d = new DCRaw;
 
+#ifndef LOCALTIME
+    putenv (const_cast<char *>("TZ=UTC"));
+#endif
     g_free(d->messageBuffer);
     d->messageBuffer = NULL;
     d->lastStatus = DCRAW_SUCCESS;
     d->verbose = 1;
     d->ifname = g_strdup(filename);
+    d->ifname_display = g_filename_display_name(d->ifname);
     if (setjmp(d->failure)) {
 	d->dcraw_message(DCRAW_ERROR,_("Fatal internal error\n"));
 	h->message = d->messageBuffer;
@@ -76,9 +81,10 @@ int dcraw_open(dcraw_data *h,char *filename)
 	return DCRAW_ERROR;
     }
     if (!(d->ifp = g_fopen (d->ifname, "rb"))) {
+	gchar *err_u8 = g_locale_to_utf8(strerror(errno), -1, NULL, NULL, NULL);
 	d->dcraw_message(DCRAW_OPEN_ERROR,_("Cannot open file %s: %s\n"),
-		filename, strerror(errno));
-	g_free(d->ifname);
+		d->ifname_display, err_u8);
+	g_free(err_u8);
 	h->message = d->messageBuffer;
 	delete d;
 	return DCRAW_OPEN_ERROR;
@@ -88,9 +94,8 @@ int dcraw_open(dcraw_data *h,char *filename)
      * to 'dcraw -i' succeeding */
     if (!d->make[0]) {
 	d->dcraw_message(DCRAW_OPEN_ERROR,_("%s: unsupported file format.\n"),
-		d->ifname);
+		d->ifname_display);
 	fclose(d->ifp);
-	g_free(d->ifname);
 	h->message = d->messageBuffer;
 	int lastStatus = d->lastStatus;
 	delete d;
@@ -98,9 +103,9 @@ int dcraw_open(dcraw_data *h,char *filename)
     }
     /* Next we check if dcraw can decode the file */
     if (!d->is_raw) {
-	d->dcraw_message(DCRAW_OPEN_ERROR,_("Cannot decode file %s\n"), d->ifname);
+	d->dcraw_message(DCRAW_OPEN_ERROR,_("Cannot decode file %s\n"),
+		d->ifname_display);
 	fclose(d->ifp);
-	g_free(d->ifname);
 	h->message = d->messageBuffer;
 	int lastStatus = d->lastStatus;
 	delete d;
@@ -203,10 +208,11 @@ int dcraw_load_raw(dcraw_data *h)
     h->raw.colors = d->colors;
     h->fourColorFilters = d->filters;
     d->dcraw_message(DCRAW_VERBOSE,_("Loading %s %s image from %s ...\n"),
-	    d->make, d->model, d->ifname);
+	    d->make, d->model, d->ifname_display);
     fseek (d->ifp, d->data_offset, SEEK_SET);
     (d->*d->load_raw)();
-    d->bad_pixels();
+    if (d->zero_is_bad) d->remove_zeroes();
+    d->bad_pixels(NULL);
     if (d->is_foveon) {
 	d->foveon_interpolate();
 	h->raw.width = h->width = d->width;
@@ -247,10 +253,12 @@ int dcraw_load_thumb(dcraw_data *h, dcraw_image_data *thumb)
     h->thumbOffset = d->thumb_offset;
     h->thumbBufferLength = d->thumb_length;
     if (d->thumb_offset==0) {
-	dcraw_message(d, DCRAW_ERROR,_("%s has no thumbnail."), d->ifname);
+	dcraw_message(d, DCRAW_ERROR,_("%s has no thumbnail."),
+		d->ifname_display);
     } else if (d->thumb_load_raw!=NULL) {
 	dcraw_message(d, DCRAW_ERROR,
-		_("Unsupported thumb format (load_raw) for %s"), d->ifname);
+		_("Unsupported thumb format (load_raw) for %s"),
+		d->ifname_display);
     } else if (d->write_thumb==&DCRaw::jpeg_thumb) {
 	h->thumbType = jpeg_thumb_type;
     } else if (d->write_thumb==&DCRaw::ppm_thumb) {
@@ -259,7 +267,7 @@ int dcraw_load_thumb(dcraw_data *h, dcraw_image_data *thumb)
 	h->thumbBufferLength = thumb->width*thumb->height*3;
     } else {
 	dcraw_message(d, DCRAW_ERROR,
-		_("Unsupported thumb format for %s"), d->ifname);
+		_("Unsupported thumb format for %s"), d->ifname_display);
     }
     h->message = d->messageBuffer;
     return d->lastStatus;
@@ -482,7 +490,7 @@ int dcraw_set_color_scale(dcraw_data *h, int useAutoWB, int useCameraWB)
 	scale_colors_INDI(h->raw.image,
 		h->rgbMax-h->black, h->black, useAutoWB, useCameraWB,
 		h->cam_mul, h->raw.height, h->raw.width, h->raw.colors,
-		h->post_mul, h->filters, d->white, d->ifname, d);
+		h->post_mul, h->filters, d->white, d->ifname_display, d);
     h->message = d->messageBuffer;
     return d->lastStatus;
 }
@@ -567,6 +575,10 @@ int dcraw_finalize_interpolate(dcraw_image_data *f, dcraw_data *h,
     int smoothPasses = 1;
     if (interpolation==dcraw_bilinear_interpolation)
 	lin_interpolate_INDI(f->image, ff, f->width, f->height, cl, d);
+#ifdef ENABLE_INTERP_NONE
+    else if (interpolation==dcraw_none_interpolation)
+        smoothing = 0;
+#endif
     else if (interpolation==dcraw_vng_interpolation)
 	vng_interpolate_INDI(f->image, ff, f->width, f->height, cl, 0xFFFF, d);
     else if (interpolation==dcraw_ahd_interpolation) {
@@ -593,7 +605,6 @@ int dcraw_finalize_interpolate(dcraw_image_data *f, dcraw_data *h,
 void dcraw_close(dcraw_data *h)
 {
     DCRaw *d = (DCRaw *)h->dcraw;
-    g_free(d->ifname);
     g_free(h->raw.image);
     delete d;
 }
