@@ -2,17 +2,21 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * dcraw_indi.c - DCRaw functions made independent
- * Copyright 2004-2007 by Udi Fuchs
+ * Copyright 2004-2008 by Udi Fuchs
  *
  * based on dcraw by Dave Coffin
  * http://www.cybercom.net/~dcoffin/
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2
- * as published by the Free Software Foundation. You should have received
- * a copy of the license along with this program.
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,9 +35,10 @@ extern const float d65_white[3];
 
 #define CLASS
 
-#define FORC3 for (c=0; c < 3; c++)
-#define FORC4 for (c=0; c < 4; c++)
-#define FORCC for (c=0; c < colors; c++)
+#define FORC(cnt) for (c=0; c < cnt; c++)
+#define FORC3 FORC(3)
+#define FORC4 FORC(4)
+#define FORCC FORC(colors)
 
 #define SQR(x) ((x)*(x))
 #define LIM(x,min,max) MAX(min,MIN(x,max))
@@ -184,7 +189,7 @@ void CLASS scale_colors_INDI(ushort (*image)[4], int maximum, const int black,
        const int use_auto_wb, const int use_camera_wb, const float cam_mul[4],
        const int iheight, const int iwidth, const int colors, float pre_mul[4],
        const unsigned filters, /*const*/ ushort white[8][8],
-       const char *ifname, void *dcraw)
+       const char *ifname_display, void *dcraw)
 {
   int dblack, row, col, x, y, c, val, sum[8];
   double dsum[8], dmin, dmax;
@@ -228,7 +233,7 @@ skip_block:
       memcpy (pre_mul, cam_mul, 4*sizeof(float));
     else
       dcraw_message (dcraw, DCRAW_NO_CAMERA_WB,
-	      _("%s: Cannot use camera white balance.\n"), ifname);
+	      _("%s: Cannot use camera white balance.\n"), ifname_display);
   }
 //  if (user_mul[0])
 //    memcpy (pre_mul, user_mul, sizeof pre_mul);
@@ -713,7 +718,8 @@ static inline int median9(int *p)
 #undef PIX_SWAP
 #undef PIX_SORT
 
-static ushort eahd_median(int row, int col, int color,
+// Just making this function inline speeds up ahd_interpolate_INDI() up to 15%
+static inline ushort eahd_median(int row, int col, int color,
     ushort (*image)[4], const int width)
 {
   //declare the pixel array
