@@ -2,12 +2,12 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw-batch.c - The standalone interface to UFRaw in batch mode.
- * Copyright 2004-2007 by Udi Fuchs
+ * Copyright 2004-2008 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2
- * as published by the Free Software Foundation. You should have received
- * a copy of the license along with this program.
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,6 +31,7 @@ int main (int argc, char **argv)
     ufraw_data *uf;
     conf_data rc, cmd, conf;
     int status;
+    int exitCode = 0;
 
     char *argFile = uf_win32_locale_to_utf8(argv[0]);
     ufraw_binary = g_path_get_basename(argFile);
@@ -73,34 +74,46 @@ int main (int argc, char **argv)
     if (optInd==argc) {
 	    ufraw_message(UFRAW_WARNING, _("No input file, nothing to do."));
     }
-    for (; optInd<argc; optInd++) {
+    int fileCount = argc - optInd;
+    int fileIndex = 1;
+    for (; optInd<argc; optInd++, fileIndex++) {
 	argFile = uf_win32_locale_to_utf8(argv[optInd]);
 	uf = ufraw_open(argFile);
 	uf_win32_locale_free(argFile);
 	if (uf==NULL) {
+	    exitCode = 1;
 	    ufraw_message(UFRAW_REPORT, NULL);
 	    continue;
 	}
 	status = ufraw_config(uf, &rc, &conf, &cmd);
 	if (status==UFRAW_ERROR) {
+	    exitCode = 1;
 	    ufraw_close(uf);
 	    g_free(uf);
 	    exit(1);
 	}
 	if (ufraw_load_raw(uf)!=UFRAW_SUCCESS) {
+	    exitCode = 1;
 	    ufraw_close(uf);
 	    g_free(uf);
 	    continue;
 	}
-	ufraw_message(UFRAW_MESSAGE, _("loaded %s"), uf->filename);
+	char stat[max_name];
+	if ( fileCount>1 )
+	    g_snprintf(stat, max_name, "[%d/%d]", fileIndex, fileCount);
+	else
+	    stat[0] = '\0';
+	ufraw_message(UFRAW_MESSAGE, _("Loaded %s%s"), uf->filename, stat);
 	if (ufraw_batch_saver(uf)==UFRAW_SUCCESS)
-	    ufraw_message(UFRAW_MESSAGE, _("saved %s"),
-		    uf->conf->outputFilename);
+	    ufraw_message(UFRAW_MESSAGE, _("Saved %s%s"),
+		    uf->conf->outputFilename, stat);
+	else
+	    exitCode = 1;
 	ufraw_close(uf);
 	g_free(uf);
     }
 //    ufraw_close(cmd.darkframe);
-    exit(0);
+    exit(exitCode);
 }
 int ufraw_batch_saver(ufraw_data *uf)
 {
