@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw-gimp.c - The GIMP plug-in.
- * Copyright 2004-2007 by Udi Fuchs
+ * Copyright 2004-2008 by Udi Fuchs
  *
  * based on the GIMP plug-in by Pawel T. Jochym jochym at ifj edu pl,
  *
@@ -10,9 +10,9 @@
  * http://www.cybercom.net/~dcoffin/
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2
- * as published by the Free Software Foundation. You should have received
- * a copy of the license along with this program.
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,6 +30,7 @@
 /* Fix some compatibility issues between CinePaint and GIMP */
 #define GIMP_CHECK_VERSION(a,b,c) 0
 typedef GimpRunModeType GimpRunMode;
+#define PLUGIN_MODE 2
 #else
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
@@ -37,6 +38,7 @@ typedef GimpRunModeType GimpRunMode;
 /* Missing and irrelevant definitions in GIMP */
 #define U16_RGB 0
 #define U16_RGB_IMAGE 0
+#define PLUGIN_MODE 1
 #endif
 #include <glib/gi18n.h>
 #include "ufraw.h"
@@ -86,7 +88,7 @@ void query()
 	    "Udi Fuchs",
 	    "Copyright 2003 by Dave Coffin\n"
 	    "Copyright 2004 by Pawel Jochym\n"
-	    "Copyright 2004-2007 by Udi Fuchs",
+	    "Copyright 2004-2008 by Udi Fuchs",
 	    "ufraw-" VERSION,
 #if GIMP_CHECK_VERSION(2,2,0)
 	    "raw image",
@@ -107,7 +109,7 @@ void query()
 	    "Loads thumbnails from digital camera raw files.",
 	    "Loads thumbnails from digital camera raw files.",
 	    "Udi Fuchs",
-	    "Copyright 2004-2007 by Udi Fuchs",
+	    "Copyright 2004-2008 by Udi Fuchs",
 	    "ufraw-" VERSION,
 	    NULL,
 	    NULL,
@@ -170,6 +172,10 @@ void run(GIMP_CONST gchar *name,
 	return;
     }
     gboolean loadThumbnail = size>0;
+    char *gtkrcfile = g_build_filename(uf_get_home_dir(),
+	".ufraw-gtkrc", NULL);
+    gtk_rc_add_default_file(gtkrcfile);
+    g_free(gtkrcfile);
     gimp_ui_init("ufraw-gimp", TRUE);
 
     uf = ufraw_open(filename);
@@ -244,7 +250,7 @@ void run(GIMP_CONST gchar *name,
 	 !loadThumbnail && !sendToGimpMode ) {
 	/* Show the preview in interactive mode, unless if we are
 	 * in thumbnail mode or 'send to gimp' mode. */
-	status = ufraw_preview(uf, 1, ufraw_save_gimp_image);
+	status = ufraw_preview(uf, PLUGIN_MODE, ufraw_save_gimp_image);
     } else {
 	if ( sendToGimpMode ) {
 	    char *text = g_strdup_printf(_("Loading raw file '%s'"),
@@ -321,7 +327,11 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
 	top = uf->conf->CropY1 * uf->image.height / uf->initialHeight;
 	left = uf->conf->CropX1 * uf->image.width / uf->initialWidth;
 #ifdef UFRAW_CINEPAINT
-	depth = 6;
+	if ( uf->conf->profile[out_profile]
+			[uf->conf->profileIndex[out_profile]].BitDepth==16 )
+	    depth = 6;
+	else
+	    depth = 3;
 #else
 	depth = 3;
 #endif
@@ -400,15 +410,16 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
 	}
     }
 #else
-    if (uf->exifBuf!=NULL) {
-	if (uf->exifBufLen>65533) {
+    ufraw_exif_prepare_output(uf);
+    if (uf->outputExifBuf!=NULL) {
+	if (uf->outputExifBufLen>65533) {
 	    ufraw_message(UFRAW_SET_WARNING,
 		    _("EXIF buffer length %d, too long, ignored.\n"),
-		    uf->exifBufLen);
+		    uf->outputExifBufLen);
 	} else {
 	    GimpParasite *exif_parasite;
 	    exif_parasite = gimp_parasite_new ("exif-data",
-		    GIMP_PARASITE_PERSISTENT, uf->exifBufLen, uf->exifBuf);
+		    GIMP_PARASITE_PERSISTENT, uf->outputExifBufLen, uf->outputExifBuf);
 	    gimp_image_parasite_attach (uf->gimpImage, exif_parasite);
 	    gimp_parasite_free (exif_parasite);
 	}
