@@ -2,20 +2,29 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw.h - Common definitions for UFRaw.
- * Copyright 2004-2007 by Udi Fuchs
+ * Copyright 2004-2008 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2
- * as published by the Free Software Foundation. You should have received
- * a copy of the license along with this program.
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  */
 
 #ifndef _UFRAW_H
 #define _UFRAW_H
 
+#ifdef HAVE_LENSFUN
+#include <lensfun.h>
+#endif /* HAVE_LENSFUN */
+
 #include "nikon_curve.h"
 #include <time.h> // for time_t
 
+/* macro to clamp a number between two values */
+#ifndef LIM
+#define LIM(x,min,max) MAX(min,MIN(x,max))
+#endif
+
 #define MAXOUT 255 /* Max output sample */
 
 #define max_curves 20
@@ -43,15 +52,15 @@ enum { linear_histogram, log_histogram };
 /* The following enum should match the dcraw_interpolation enum
  * in dcraw_api.h. */
 enum { ahd_interpolation, vng_interpolation, four_color_interpolation,
-       ppg_interpolation, bilinear_interpolation, half_interpolation,
-       obsolete_eahd_interpolation, num_interpolations };
+       ppg_interpolation, bilinear_interpolation, none_interpolation,
+       half_interpolation, obsolete_eahd_interpolation, num_interpolations };
 enum { no_id, also_id, only_id, send_id };
 enum { manual_curve, linear_curve, custom_curve, camera_curve };
 enum { in_profile, out_profile, display_profile, profile_types};
 enum { raw_expander, live_expander, expander_count };
-enum { ppm8_type, ppm16_type, tiff8_type, tiff16_type, jpeg_type,
-       png8_type, png16_type, embedded_jpeg_type, embedded_png_type,
-       fits_type};
+enum { ppm_type, ppm16_deprecated_type, tiff_type, tiff16_deprecated_type,
+       jpeg_type, png_type, png16_deprecated_type,
+       embedded_jpeg_type, embedded_png_type, fits_type, num_types };
 enum { clip_details, restore_lch_details, restore_hsv_details,
        restore_types };
 enum { digital_highlights, film_highlights, highlights_types };
@@ -59,8 +68,10 @@ typedef enum { display_developer, file_developer, auto_developer }
 	DeveloperMode;
 typedef enum { perceptual_intent, relative_intent, saturation_intent,
 	absolute_intent, disable_intent } Intent;
-typedef enum { ufraw_first_phase, ufraw_denoise_phase, ufraw_final_phase,
-	ufraw_phases_num } UFRawPhase;
+typedef enum { ufraw_first_phase, ufraw_denoise_phase, ufraw_develop_phase,
+        ufraw_lensfun_phase, ufraw_phases_num } UFRawPhase;
+typedef enum { grayscale_none, grayscale_lightness, grayscale_luminance,
+	grayscale_value, grayscale_mixer } GrayscaleMode;
 
 typedef struct {
     const char *make;
@@ -82,11 +93,16 @@ typedef struct {
     gboolean updateTransform;
     void *colorTransform;
     double saturation;
+#ifdef UFRAW_CONTRAST
+    double contrast;
+#endif
     CurveData baseCurveData, luminosityCurveData;
     guint16 gammaCurve[0x10000];
     void *luminosityProfile;
     void *TransferFunction[3];
     void *saturationProfile;
+    GrayscaleMode grayscaleMode;
+    double grayscaleMixer[3];
 } developer_data;
 
 typedef guint16 image_type[4];
@@ -96,7 +112,7 @@ typedef struct {
     char file[max_path];
     char productName[max_name];
     double gamma, linear;
-    gboolean useMatrix;
+    int BitDepth;
 } profile_data;
 
 /* conf_data holds the configuration data of UFRaw.
@@ -138,6 +154,9 @@ typedef struct {
     double temperature, green;
     double chanMul[4];
     double threshold;
+#ifdef UFRAW_CONTRAST
+    double contrast;
+#endif
     double exposure, saturation, black; /* black is only used in CMD */
     int ExposureNorm;
     int restoreDetails, clipHighlights;
@@ -154,6 +173,9 @@ typedef struct {
     char darkframeFile[max_path];
     struct ufraw_struct *darkframe;
     int CropX1, CropY1, CropX2, CropY2;
+    double rotationAngle;
+    int grayscaleMode;
+    double grayscaleMixer[3];
 
     /* SAVE options */
     char inputFilename[max_path], outputFilename[max_path],
@@ -162,15 +184,18 @@ typedef struct {
     int type, compression, createID, embedExif, progressiveJPEG;
     int shrink, size;
     gboolean overwrite, losslessCompress, embeddedImage;
+    gboolean rotate;
 
     /* GUI settings */
     double Zoom;
     int Scale;
+    gboolean LockAspect; /* True if aspect ratio is locked */
     int saveConfiguration;
     int histogram, liveHistogramScale;
     int rawHistogramScale;
     int expander[expander_count];
     gboolean overExp, underExp, blinkOverUnder;
+    gboolean RememberOutputPath;
     gboolean WindowMaximized;
     char curvePath[max_path];
     char profilePath[max_path];
@@ -179,18 +204,37 @@ typedef struct {
 
     /* EXIF data */
     int orientation;
-    float iso_speed, shutter, aperture, focal_len;
+    float iso_speed, shutter, aperture, focal_len, subject_distance;
     char exifSource[max_name], isoText[max_name], shutterText[max_name],
 	 apertureText[max_name], focalLenText[max_name],
 	 focalLen35Text[max_name], lensText[max_name],
 	 flashText[max_name], whiteBalanceText[max_name];
     char timestampText[max_name], make[max_name], model[max_name];
     time_t timestamp;
+    /* Unfortunately dcraw strips make and model, but we need originals too */
+    char real_make[max_name], real_model[max_name];
+
+#ifdef HAVE_LENSFUN
+    lfDatabase *lensdb; /* mount/camera/lens database */
+    lfCamera *camera; /* camera description */
+    lfLens *lens; /* lens description */
+    lfLensCalibDistortion lens_distortion; /* lens distortion parameters */
+    lfLensCalibTCA lens_tca; /* lens tca parameters */
+    lfLensCalibVignetting lens_vignetting; /* lens vignetting parameters */
+    lfLensType cur_lens_type;
+    float lens_scale; /* Additional lens postprocessing scale power-of-two, default 0 */
+#endif /* HAVE_LENSFUN */
 } conf_data;
 
 typedef struct {
     guint8 *buffer;
     int height, width, depth, rowstride;
+    /* This bit field marks valid pieces of the image with 1's.
+       The variable contains a fixed 4x8 matrix of bits, every bit containing
+       the validity of the respective subarea of the whole image. The subarea
+       sizes are determined by dividing the width by 4 and height by 8.
+       This field must always contain at least 32 bits. */
+    long valid;
 } ufraw_image_data;
 
 /* image_data should be phased out and replaced by ufraw_image_data */
@@ -205,6 +249,7 @@ typedef struct ufraw_struct {
     char *message;
     char filename[max_path];
     int initialHeight, initialWidth, rgbMax, colors, raw_color, useMatrix;
+    int rotatedHeight, rotatedWidth;
     gboolean LoadingID; /* Indication that we are loading an ID file */
     float rgb_cam[3][4];
     int ConvertShrink;
@@ -214,17 +259,23 @@ typedef struct ufraw_struct {
     void *raw;
     gboolean HaveFilters;
     void *unzippedBuf;
-    unsigned unzippedBufLen;
+    gsize unzippedBufLen;
     developer_data *developer;
     developer_data *AutoDeveloper;
     conf_data *conf;
     void *widget;
-    guchar *exifBuf;
-    guint exifBufLen;
+    guchar *inputExifBuf;
+    guint inputExifBufLen;
+    guchar *outputExifBuf;
+    guint outputExifBufLen;
     int gimpImage;
-    int *RawLumHistogram;
-    int RawLumChanMul[4];
-    int RawLumCount;
+    int *RawHistogram;
+    int RawChanMul[4];
+    int RawCount;
+#ifdef HAVE_LENSFUN
+    int postproc_ops; /* postprocessing operations (LF_MODIFY_XXX) */
+    lfModifier *modifier;
+#endif
 } ufraw_data;
 
 extern const conf_data conf_default;
@@ -245,16 +296,22 @@ int ufraw_load_darkframe(ufraw_data *uf);
 void ufraw_developer_prepare(ufraw_data *uf, DeveloperMode mode);
 int ufraw_convert_image(ufraw_data *uf);
 int ufraw_convert_image_init(ufraw_data *uf);
-int ufraw_convert_image_first_phase(ufraw_data *uf);
+int ufraw_convert_image_first_phase(ufraw_data *uf, gboolean lensfix);
 int ufraw_convert_image_init_phase(ufraw_data *uf);
-int ufraw_convert_image_area(ufraw_data *uf,
-	int x, int y, int width, int height, UFRawPhase fromPhase);
+ufraw_image_data *ufraw_convert_image_area (ufraw_data *uf, int saidx, UFRawPhase phase);
 void ufraw_close(ufraw_data *uf);
+void ufraw_flip_orientation(ufraw_data *uf, int flip);
 int ufraw_flip_image(ufraw_data *uf, int flip);
 int ufraw_set_wb(ufraw_data *uf);
 void ufraw_auto_expose(ufraw_data *uf);
 void ufraw_auto_black(ufraw_data *uf);
 void ufraw_auto_curve(ufraw_data *uf);
+void ufraw_rotate_row(image_data *image, void *pixbuf, double angle,
+		      int bitDepth, int row, int offset, int width);
+
+void ufraw_img_get_subarea_coord (ufraw_image_data *img, unsigned saidx,
+                                  int *x, int *y, int *w, int *h);
+unsigned ufraw_img_get_subarea_idx (ufraw_image_data *img, int x, int y);
 
 /* prototypes for functions in ufraw_message.c */
 char *ufraw_get_message(ufraw_data *uf);
@@ -291,12 +348,20 @@ void RGB_to_Temperature(double RGB[3], double *T, double *Green);
 int curve_load(CurveData *cp, char *filename);
 int curve_save(CurveData *cp, char *filename);
 char *curve_buffer(CurveData *cp);
+/* Useful functions for handling the underappreciated Glib ptr arrays */
+int ptr_array_insert_sorted (GPtrArray *array, const void *item, GCompareFunc compare);
+int ptr_array_find_sorted (const GPtrArray *array, const void *item, GCompareFunc compare);
+void ptr_array_insert_index (GPtrArray *array, const void *item, int index);
 
 /* prototypes for functions in ufraw_conf.c */
 int conf_load(conf_data *c, const char *confFilename);
 int conf_save(conf_data *c, char *confFilename, char **confBuffer);
+/* copy default config to given instance and initialize non-const fields */
+void conf_init (conf_data *c);
 /* Copy the image manipulation options from *src to *dst */
 void conf_copy_image(conf_data *dst, const conf_data *src);
+/* Copy the transformation options from *src to *dst */
+void conf_copy_transform(conf_data *dst, const conf_data *src);
 /* Copy the 'save options' from *src to *dst */
 void conf_copy_save(conf_data *dst, const conf_data *src);
 int conf_set_cmd(conf_data *conf, const conf_data *cmd);
@@ -316,7 +381,6 @@ void develope(void *po, guint16 pix[4], developer_data *d, int mode,
 void develop_linear(guint16 in[4], guint16 out[3], developer_data *d);
 
 /* prototype for functions in ufraw_saver.c */
-long ufraw_save_as(ufraw_data *uf, void *widget);
 long ufraw_save_now(ufraw_data *uf, void *widget);
 long ufraw_send_to_gimp(ufraw_data *uf);
 
@@ -338,7 +402,9 @@ void ufraw_chooser(conf_data *conf, const char *defPath);
 void ufraw_icons_init();
 
 /* prototype for functions in ufraw_exiv2.cc */
-int ufraw_exif_from_exiv2(ufraw_data *uf);
+int ufraw_exif_read_input(ufraw_data *uf);
+int ufraw_exif_prepare_output(ufraw_data *uf);
+int ufraw_exif_write(ufraw_data *uf);
 
 /* status numbers from DCRaw and UFRaw */
 #define UFRAW_SUCCESS 0
