@@ -2,12 +2,12 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_developer.c - functions for developing images or more exactly pixels.
- * Copyright 2004-2007 by Udi Fuchs
+ * Copyright 2004-2008 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2
- * as published by the Free Software Foundation. You should have received
- * a copy of the license along with this program.
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,6 +40,9 @@ developer_data *developer_init()
     d->gamma = -1;
     d->linear = -1;
     d->saturation = -1;
+#ifdef UFRAW_CONTRAST
+    d->contrast = -1;
+#endif
     for (i=0; i<profile_types; i++) {
 	d->profile[i] = NULL;
 	strcpy(d->profileFile[i],"no such file");
@@ -57,6 +60,8 @@ developer_data *developer_init()
     d->intent[display_profile] = -1;
     d->updateTransform = TRUE;
     d->colorTransform = NULL;
+    d->grayscaleMode = -1;
+    d->grayscaleMixer[0] = d->grayscaleMixer[1] = d->grayscaleMixer[2] = -1;
     cmsSetErrorHandler(lcms_message);
     return d;
 }
@@ -142,36 +147,60 @@ void developer_display_profile(developer_data *d,
     }
 }
 
-#define CLAMP_AB_DOUBLE(ab) \
-	( (ab<-128.0) ? -128.0 : ( (ab>127.9961) ? +127.9961 : ab) )
+static double clamp(double in, double min, double max)
+{
+    return (in < min) ? min : (in > max) ? max : in;
+}
 
-static int saturation_sampler(register WORD In[], register WORD Out[],
-	register LPVOID Cargo)
+struct contrast_saturation
+{
+  double contrast;
+  double saturation;
+};
+
+/* Scale in along a curve from min to max by scale */
+static double scale_curve(double in, double min, double max, double scale)
+{
+  double halfrange = (max - min) / 2.0;
+  /* Normalize in to [ -1, 1 ] */
+  double value = clamp((in - min) / halfrange - 1.0, -1.0, 1.0);
+  /* Linear scaling makes more visual sense for low contrast values. */
+  if (scale > 1.0) {
+    double n = fabs(value);
+    if (n > 1.0)
+      n = 1.0;
+    scale = n <= 0.0 ? 0.0 : ( 1.0 - pow( 1.0 - n, scale ) ) / n;
+  }
+  return clamp((value * scale + 1.0) * halfrange + min, min, max);
+}
+
+static const double max_luminance = 100.0;
+static const double max_colorfulness = 181.019336; /* sqrt(128*128+128*128) */
+
+static int contrast_saturation_sampler(WORD In[], WORD Out[], LPVOID Cargo)
 {
     cmsCIELab Lab;
-    double saturation = *(double *)Cargo;
+    cmsCIELCh LCh;
+    const struct contrast_saturation* cs = Cargo;
 
     cmsLabEncoded2Float(&Lab, In);
-
-    if ( Lab.a!=0.0 || Lab.b!=0.0 ) {
-
-	/* Normalized Chroma of current color (0.0 to 1.0) */
-	double Cn = MAX( fabs(Lab.a), fabs(Lab.b) ) / 128.0;
-
-	double scale = ( 1.0 - pow( 1.0 - Cn , saturation ) ) / Cn;
-	Lab.a = CLAMP_AB_DOUBLE(Lab.a * scale);
-	Lab.b = CLAMP_AB_DOUBLE(Lab.b * scale);
-    }
+    cmsLab2LCh(&LCh, &Lab);
+    LCh.L = scale_curve(LCh.L, 0.0, max_luminance, cs->contrast);
+    LCh.C = scale_curve(LCh.C, -max_colorfulness, max_colorfulness,
+			cs->saturation);
+    cmsLCh2Lab(&Lab, &LCh);
     cmsFloat2LabEncoded(Out, &Lab);
 
     return TRUE;
 }
 
 /* Based on lcms' cmsCreateBCHSWabstractProfile() */
-static cmsHPROFILE create_saturation_profile(double saturation)
+static cmsHPROFILE create_contrast_saturation_profile(double contrast,
+						      double saturation)
 {
     cmsHPROFILE hICC;
     LPLUT Lut;
+    struct contrast_saturation cs = { contrast, saturation };
 
     hICC = _cmsCreateProfilePlaceholder();
     if (hICC==NULL) return NULL;// can't allocate
@@ -183,8 +212,8 @@ static cmsHPROFILE create_saturation_profile(double saturation)
 
     // Creates a LUT with 3D grid only
     Lut = cmsAllocLUT();
-    cmsAlloc3DGrid(Lut, 7, 3, 3);
-    if (!cmsSample3DGrid(Lut, saturation_sampler, &saturation , 0)) {
+    cmsAlloc3DGrid(Lut, 11, 3, 3);
+    if (!cmsSample3DGrid(Lut, contrast_saturation_sampler, &cs , 0)) {
 	// Shouldn't reach here
 	cmsFreeLUT(Lut);
 	cmsCloseProfile(hICC);
@@ -265,7 +294,7 @@ static void developer_create_transform(developer_data *d, DeveloperMode mode)
 		    d->profile[in_profile], TYPE_RGB_16,
 		    d->profile[display_profile], TYPE_RGB_16,
 		    d->profile[out_profile],
-		    d->intent[display_profile], d->intent[out_profile],
+		    d->intent[out_profile], d->intent[display_profile],
 		    cmsFLAGS_SOFTPROOFING);
 	} else {
 	    /* Following code imitates the function
@@ -280,7 +309,7 @@ static void developer_create_transform(developer_data *d, DeveloperMode mode)
 		prof[i++] = d->saturationProfile;
 	    d->colorTransform = cmsCreateMultiprofileTransform(prof, i,
 		    TYPE_RGB_16, NOCOLORSPACECHECK(TYPE_RGB_16),
-		    d->intent[display_profile], cmsFLAGS_SOFTPROOFING);
+		    d->intent[out_profile], cmsFLAGS_SOFTPROOFING);
 
 	    prof[0] = cmsTransform2DeviceLink(d->colorTransform,
 		    cmsFLAGS_GUESSDEVICECLASS);
@@ -289,7 +318,7 @@ static void developer_create_transform(developer_data *d, DeveloperMode mode)
 		    prof[0], TYPE_RGB_16,
 		    d->profile[display_profile], TYPE_RGB_16,
 		    d->profile[out_profile],
-		    d->intent[display_profile], d->intent[out_profile],
+		    d->intent[out_profile], d->intent[display_profile],
 		    cmsFLAGS_SOFTPROOFING);
 	}
     }
@@ -302,6 +331,7 @@ void developer_prepare(developer_data *d, conf_data *conf,
     unsigned c, i;
     profile_data *in, *out, *display;
     CurveData *baseCurve, *curve;
+    double total;
 
     if ( mode!=d->mode ) {
 	d->mode = mode;
@@ -337,12 +367,32 @@ void developer_prepare(developer_data *d, conf_data *conf,
 	    for (c=0; c<d->colors; c++)
 		d->colorMatrix[i][c] = rgb_cam[i][c]*0x10000;
 
+    switch (conf->grayscaleMode) {
+
+    case grayscale_mixer:
+        d->grayscaleMode = grayscale_mixer;
+	for (c = 0, total = 0.0; c < 3; ++c)
+	    total += fabs(conf->grayscaleMixer[c]);
+	total = total == 0.0 ? 1.0 : total;
+	for (c = 0; c < 3; ++c)
+	    d->grayscaleMixer[c] = conf->grayscaleMixer[c] / total;
+	break;
+
+    case grayscale_lightness:
+    case grayscale_value:
+        d->grayscaleMode = conf->grayscaleMode;
+	break;
+
+    default:
+        d->grayscaleMode = grayscale_none;
+    }
+
     d->restoreDetails = conf->restoreDetails;
     int clipHighlights = conf->clipHighlights;
     unsigned exposure = pow(2, conf->exposure) * 0x10000;
     /* Handle the exposure normalization for Canon EOS cameras. */
     if ( conf->ExposureNorm>0 )
-	exposure = exposure * d->rgbMax / conf->ExposureNorm;
+	exposure = (guint64)exposure * d->rgbMax / conf->ExposureNorm;
     if ( exposure>=0x10000 ) d->restoreDetails = clip_details;
     if ( exposure<=0x10000 ) clipHighlights = digital_highlights;
     /* Check if gamma curve data has changed. */
@@ -440,13 +490,31 @@ void developer_prepare(developer_data *d, conf_data *conf,
 	}
 	d->updateTransform = TRUE;
     }
-    if ( conf->saturation!=d->saturation ) {
-	d->saturation = conf->saturation;
+    if ( conf->saturation!=d->saturation
+#ifdef UFRAW_CONTRAST
+	 || conf->contrast!=d->contrast
+#endif
+         || conf->grayscaleMode == grayscale_luminance ) {
+#ifdef UFRAW_CONTRAST
+        d->contrast = conf->contrast;
+#endif
+	d->saturation = (conf->grayscaleMode == grayscale_luminance)
+	    ? 0 : conf->saturation;
 	cmsCloseProfile(d->saturationProfile);
-	if (d->saturation==1)
+	if (d->saturation==1.0
+#ifdef UFRAW_CONTRAST
+	    && d->contrast==1.0
+#endif
+	   )
 	    d->saturationProfile = NULL;
 	else
-	    d->saturationProfile = create_saturation_profile(d->saturation);
+	    d->saturationProfile = create_contrast_saturation_profile(
+#ifdef UFRAW_CONTRAST
+		    d->contrast,
+#else
+		    1.0,
+#endif
+		    d->saturation);
 	d->updateTransform = TRUE;
     }
     developer_create_transform(d, mode);
@@ -552,6 +620,44 @@ inline void develope(void *po, guint16 pix[4], developer_data *d, int mode,
     else for (i=0; i<3*count; i++) p8[i] = buf[i] >> 8;
 }
 
+static void develop_grayscale(guint16 *pixel, const developer_data *d)
+{
+    gint32 spot;
+    guint16 min;
+    guint16 max;
+
+    switch (d->grayscaleMode) {
+
+    case grayscale_mixer:
+        spot = pixel[0] * d->grayscaleMixer[0]
+	  + pixel[1] * d->grayscaleMixer[1]
+	  + pixel[2] * d->grayscaleMixer[2];
+	if (spot > 65535) spot = 65535;
+	else if (spot < 0) spot = 0;
+	break;
+
+    case grayscale_lightness:
+        min = max = pixel[0];
+	if (pixel[1] > max) max = pixel[1];
+	if (pixel[2] > max) max = pixel[2];
+	if (pixel[1] < min) min = pixel[1];
+	if (pixel[2] < min) min = pixel[2];
+	spot = ((int)min + (int)max) / 2;
+	break;
+
+    case grayscale_value:
+        max = pixel[0];
+	if (pixel[1] > max) max = pixel[1];
+	if (pixel[2] > max) max = pixel[2];
+	spot = max;
+	break;
+
+    default:
+        return;
+    }
+    pixel[0] = pixel[1] = pixel[2] = spot;
+}
+
 void develop_linear(guint16 in[4], guint16 out[3], developer_data *d)
 {
     unsigned c, cc;
@@ -670,4 +776,5 @@ void develop_linear(guint16 in[4], guint16 out[3], developer_data *d)
     }
     for (c=0; c<3; c++)
 	out[c] = MIN(MAX(tmppix[c], 0), 0xFFFF);
+    develop_grayscale(out, d);
 }
