@@ -2,12 +2,12 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_routines.c - general routines
- * Copyright 2004-2007 by Udi Fuchs
+ * Copyright 2004-2008 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2
- * as published by the Free Software Foundation. You should have received
- * a copy of the license along with this program.
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -63,13 +63,15 @@ void uf_init_locale(const char *exename)
 #ifdef WIN32
 	/* In Windows the localedir is found relative to the exe file.
 	 * The exact location here should match ufraw-setup.iss.in */
-	if ( strcasecmp(g_path_get_basename(exename), "ufraw-gimp.exe")==0 ) {
+	char *basename = g_path_get_basename(exename);
+	if ( strcasecmp(basename, "ufraw-gimp.exe")==0 ) {
 	    localedir = g_strconcat(g_path_get_dirname(exename),
 		    "/../../../locale", NULL);
 	} else {
 	    localedir = g_strconcat(g_path_get_dirname(exename),
 		    "/../lib/locale", NULL);
 	}
+	g_free(basename);
 #else
 	exename = exename; /* suppress warning */
 	/* In other environments localedir is set at compile time */
@@ -113,15 +115,25 @@ char *uf_file_set_absolute(const char *filename)
 	return g_strdup(filename);
     } else {
 #ifdef HAVE_CANONICALIZE_FILE_NAME
+	// canonicalize_file_name() requires the file to exist.
+	// This is why we need to split 'filename' to dirname and basename.
 	char *path = g_path_get_dirname(filename);
-	char *base = g_path_get_basename(filename);
 	char *canon = canonicalize_file_name(path);
-	char *abs;
-	if ( canon==NULL )
-	    abs = g_strdup(filename);
-	else
-	    abs = g_build_filename(canon, base, NULL);
+	if ( canon==NULL ) {
+	    // We should never reach this code
+	    g_message("Error in canonicalize_file_name(""%s""): %s",
+		    path, strerror(errno));
+	    g_free(path);
+	    return g_strdup(filename);
+	}
+	// If filename ends with a separator there is no basename
+	if ( strlen(path)==strlen(filename)-1 ) {
+	    g_free(path);
+	    return canon;
+	}
 	g_free(path);
+	char *base = g_path_get_basename(filename);
+	char *abs = g_build_filename(canon, base, NULL);
 	g_free(base);
 	g_free(canon);
 	return abs;
@@ -154,13 +166,12 @@ char *uf_markup_buf(char *buffer, const char *format, ...)
     }
 }
 
-const char raw_ext[]= "arw,bay,bmq,cr2,crw,cs1,dc2,dcr,dng,erf,fff,hdr,ia,jpg,"
-	"k25,kc2,kdc,mdc,mef,mos,mrw,nef,orf,pef,pxn,qtk,raf,raw,rdc,sr2,srf,"
-	"sti,tif,ufraw,x3f";
+const char raw_ext[]= "3fr,arw,bay,bmq,cine,cr2,crw,cs1,dc2,dcr,dng,erf,fff,"
+	"hdr,ia,jpg,k25,kc2,kdc,mdc,mef,mos,mrw,nef,nrw,orf,pef,pxn,qtk,raf,"
+	"raw,rdc,rw2,sr2,srf,sti,tif,ufraw,x3f";
 
 const char *file_type[] = { ".ppm", ".ppm", ".tif", ".tif", ".jpg",
-	".png", ".png", ".embedded.jpg", ".embedded.png",
-    ".fits" };
+	".png", ".png", ".embedded.jpg", ".embedded.png", ".fits" };
 
 /* Set locale of LC_NUMERIC to "C" to make sure that printf behaves correctly.*/
 char *uf_set_locale_C()
@@ -200,7 +211,8 @@ void uf_reset_locale(char *locale)
 
 double profile_default_linear(profile_data *p)
 {
-    if ( !strcmp(p->name, "sRGB") )
+    if ( !strcmp(p->name, "No profile")
+       || !strcmp(p->name, "Color matrix") )
 	return 0.1;
     else
 	return 0.0;
@@ -208,7 +220,8 @@ double profile_default_linear(profile_data *p)
 
 double profile_default_gamma(profile_data *p)
 {
-    if ( !strcmp(p->name, "sRGB") )
+    if ( !strcmp(p->name, "No profile")
+       || !strcmp(p->name, "Color matrix") )
 	return 0.45;
     else if ( !strncmp(p->productName, "Nikon DBase for NEF", 19) /*For D100*/
 	    || !strncmp(p->productName, "Nikon D1 for NEF", 16)
@@ -225,7 +238,7 @@ double profile_default_gamma(profile_data *p)
 
 /* Convert between Temperature and RGB.
  * Base on information from http://www.brucelindbloom.com/
- * The fit for D-illuminant between 4000K and 12000K are from CIE
+ * The fit for D-illuminant between 4000K and 15000K are from CIE
  * The generalization to 2000K < T < 4000K and the blackbody fits
  * are my own and should be taken with a grain of salt.
  */
@@ -271,7 +284,7 @@ void RGB_to_Temperature(double RGB[3], double *T, double *Green)
 {
     double Tmax, Tmin, testRGB[3];
     Tmin = 2000;
-    Tmax = 12000;
+    Tmax = 15000;
     for (*T=(Tmax+Tmin)/2; Tmax-Tmin>10; *T=(Tmax+Tmin)/2) {
 	Temperature_to_RGB(*T, testRGB);
 	if (testRGB[2]/testRGB[0] > RGB[2]/RGB[0])
@@ -343,15 +356,25 @@ static void curve_parse_text(GMarkupParseContext *context, const gchar *text,
     }
     /* A negative gamma marks that we are in a Curve XML block */
     if (c->m_gamma < 0) {
-	if (!strcmp("MinXY", element))
+	if (!strcmp("MinXY", element)) {
 	    sscanf(temp, "%lf %lf", &c->m_min_x, &c->m_min_y);
-	if (!strcmp("MaxXY", element))
+	    c->m_min_x = LIM(c->m_min_x, 0, 1);
+	    c->m_min_y = LIM(c->m_min_y, 0, 1);
+	}
+	if (!strcmp("MaxXY", element)) {
 	    sscanf(temp, "%lf %lf", &c->m_max_x, &c->m_max_y);
+	    c->m_max_x = LIM(c->m_max_x, 0, 1);
+	    c->m_max_y = LIM(c->m_max_y, 0, 1);
+	}
 	if (!strcmp("AnchorXY", element)) {
 	    /* If one anchor is supplied then all anchors should be supplied */
 	    sscanf(temp, "%lf %lf",
 		    &c->m_anchors[c->m_numAnchors].x,
 		    &c->m_anchors[c->m_numAnchors].y);
+	    c->m_anchors[c->m_numAnchors].x = 
+	     	LIM(c->m_anchors[c->m_numAnchors].x, 0, 1);
+	    c->m_anchors[c->m_numAnchors].y = 
+	     	LIM(c->m_anchors[c->m_numAnchors].y, 0, 1);
 	    c->m_numAnchors++;
 	}
     }
@@ -410,8 +433,7 @@ int curve_load(CurveData *cp, char *filename)
     }
     char *base = g_path_get_basename(filename);
     char *name = uf_file_set_type(base, "");
-    char *utf8 = g_filename_to_utf8(name, -1, NULL, NULL, NULL);
-    if (utf8==NULL) utf8 = g_strdup("Unknown file name");
+    char *utf8 = g_filename_display_name(name);
     g_strlcpy(cp->name, utf8, max_name);
     g_free(utf8);
     g_free(name);
@@ -457,8 +479,7 @@ int curve_save(CurveData *cp, char *filename)
 	fprintf(out, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
 	char *base = g_path_get_basename(filename);
 	char *name = uf_file_set_type(base, "");
-	char *utf8 = g_filename_to_utf8(name, -1, NULL, NULL, NULL);
-	if (utf8==NULL) utf8 = strdup("Unknown file name");
+	char *utf8 = g_filename_display_name(name);
 	fprintf(out, "<Curve Version='%d'>%s\n", conf_default.version, utf8);
 	g_free(utf8);
 	g_free(name);
@@ -498,3 +519,83 @@ char *curve_buffer(CurveData *c)
     }
     return buf;
 }
+
+int ptr_array_insert_sorted (
+    GPtrArray *array, const void *item, GCompareFunc compare)
+{
+    int length = array->len;
+    g_ptr_array_set_size (array, length + 1);
+    const void **root = (const void **)array->pdata;
+
+    int m = 0, l = 0, r = length - 1;
+
+    // Skip trailing NULL, if any
+    if (l <= r && !root [r])
+        r--;
+    
+    while (l <= r)
+    {
+        m = (l + r) / 2;
+        int cmp = compare (root [m], item);
+
+        if (cmp == 0)
+        {
+            ++m;
+            goto done;
+        }
+        else if (cmp < 0)
+            l = m + 1;
+        else
+            r = m - 1;
+    }
+    if (r == m)
+        m++;
+
+done:
+    memmove (root + m + 1, root + m, (length - m) * sizeof (void *));
+    root [m] = item;
+    return m;
+}
+
+int ptr_array_find_sorted (
+    const GPtrArray *array, const void *item, GCompareFunc compare)
+{
+    int length = array->len;
+    void **root = array->pdata;
+
+    int l = 0, r = length - 1;
+    int m = 0, cmp = 0;
+
+    if (!length)
+        return -1;
+
+    // Skip trailing NULL, if any
+    if (!root [r])
+        r--;
+
+    while (l <= r)
+    {
+        m = (l + r) / 2;
+        cmp = compare (root [m], item);
+
+        if (cmp == 0)
+            return m;
+        else if (cmp < 0)
+            l = m + 1;
+        else
+            r = m - 1;
+    }
+    
+    return -1;
+}
+
+void ptr_array_insert_index (
+    GPtrArray *array, const void *item, int index)
+{
+    const void **root;
+    int length = array->len;
+    g_ptr_array_set_size (array, length + 1);
+    root = (const void **)array->pdata;
+    memmove (root + index + 1, root + index, (length - index) * sizeof (void *));
+    root [index] = item;
+}
