@@ -2,12 +2,12 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_writer.c - functions to output image files in different formats.
- * Copyright 2004-2007 by Udi Fuchs
+ * Copyright 2004-2008 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2
- * as published by the Free Software Foundation. You should have received
- * a copy of the license along with this program.
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,6 +35,58 @@
 #include <fitsio.h>
 #endif
 
+static void grayscale_8bit(guint8 *graybuf, const guint8 *pixbuf, int width)
+{
+    int i;
+    for (i = 0; i < width; ++i, ++graybuf, pixbuf += 3)
+        *graybuf = pixbuf[1];
+}
+
+static void grayscale_16bit(guint16 *graybuf, const guint16 *pixbuf, int width)
+{
+    int i;
+    for (i = 0; i < width; ++i, ++graybuf, pixbuf += 3)
+        *graybuf = pixbuf[1];
+}
+
+static int ppm8_row_writer(
+    ufraw_data *uf,
+    void * volatile out,
+    void * pixbuf,
+    int row, int width, int grayscale)
+{
+    row=row;
+    if ((int)fwrite(pixbuf, grayscale ? 1 : 3, width, out)<width) {
+	ufraw_set_error(uf, _("Error creating file '%s'."),
+        uf->conf->outputFilename);
+	ufraw_set_error(uf, g_strerror(errno));
+	return UFRAW_ERROR;
+    } else {
+	return UFRAW_SUCCESS;
+    }
+}
+
+static int ppm16_row_writer(
+    ufraw_data *uf,
+    void * volatile out,
+    void * pixbuf,
+    int row, int width, int grayscale)
+{
+    row=row;
+    guint16 *pixbuf16 = (guint16 *) pixbuf;
+    int i;
+    for (i=0; i<3*width; i++)
+	pixbuf16[i] = g_htons(pixbuf16[i]);
+    if ((int)fwrite(pixbuf16, grayscale ? 2 : 6, width, out)<width) {
+	ufraw_set_error(uf, _("Error creating file '%s'."),
+	uf->conf->outputFilename);
+	ufraw_set_error(uf, g_strerror(errno));
+	return UFRAW_ERROR;
+    } else {
+	return UFRAW_SUCCESS;
+    }
+}
+
 #ifdef HAVE_LIBTIFF
 // There seem to be no way to get the libtiff message without a static variable
 // Therefore the folloing code is not thread-safe.
@@ -45,6 +97,24 @@ static void tiff_messenger(const char *module, const char *fmt, va_list ap)
     (void)module;
     vsnprintf(ufraw_tiff_message, max_path, fmt, ap);
 }
+
+int tiff_row_writer(
+    ufraw_data *uf,
+    void * volatile out,
+    void * pixbuf,
+    int row, int width, int grayscale)
+{
+    width=width; grayscale=grayscale;
+    if (TIFFWriteScanline(out, pixbuf, row, 0)<0) {
+	// 'errno' does seem to contain useful information
+	ufraw_set_error(uf, _("Error creating file."));
+	ufraw_set_error(uf, ufraw_tiff_message);
+	ufraw_tiff_message[0] = '\0';
+	return UFRAW_ERROR;
+    } else {
+	return UFRAW_SUCCESS;
+    }
+}
 #endif /*HAVE_LIBTIFF*/
 
 #ifdef HAVE_LIBJPEG
@@ -81,6 +151,21 @@ static void jpeg_error_handler(j_common_ptr cinfo)
 	    cinfo->err->msg_parm.i[2],
 	    cinfo->err->msg_parm.i[3]);
 }
+
+static int jpeg_row_writer(
+    ufraw_data *uf,
+    void * volatile out,
+    void * pixbuf,
+    int row, int width, int grayscale)
+{
+    row=row; width=width; grayscale=grayscale;
+    guint8 *pixbuf8 = pixbuf;
+    jpeg_write_scanlines((struct jpeg_compress_struct *)out, &pixbuf8, 1);
+    if ( ufraw_is_error(uf) )
+	return UFRAW_ERROR;
+    else
+	return UFRAW_SUCCESS;
+}
 #endif /*HAVE_LIBJPEG*/
 
 #ifdef HAVE_LIBPNG
@@ -102,8 +187,96 @@ static void png_warning_handler(png_structp png,
 static void PNGwriteRawProfile(png_struct *ping,
     png_info *ping_info, char *profile_type, guint8 *profile_data,
     png_uint_32 length);
+
+int png_row_writer(
+    ufraw_data *uf,
+    void * volatile out,
+    void * pixbuf,
+    int row, int width, int grayscale)
+{
+    uf=uf; row=row; width=width; grayscale=grayscale;
+
+    png_write_row(out, (guint8 *)pixbuf);
+
+    return UFRAW_SUCCESS;
+}
 #endif /*HAVE_LIBPNG*/
 
+void write_image_data(
+    ufraw_data *uf,
+    void * volatile out,
+    int width, int height, int left, int top, int bitDepth, int grayscaleMode,
+    int (*row_writer) (ufraw_data *, void * volatile, void *, int, int, int))
+{
+    int row, rowStride;
+    image_type *rawImage;
+    rowStride = uf->image.width;
+    rawImage = uf->image.image;
+    guint16 *graybuf;
+    guint16 *pixbuf;
+    guint16 *finalbuf;
+
+    pixbuf = g_new(guint16, width*3);
+    graybuf = g_new(guint16, width);
+    finalbuf = grayscaleMode ? graybuf : pixbuf;
+
+    if (uf->conf->rotationAngle != 0) {
+	// Buffer for unrotated image.
+	image_data image;
+	image.width = uf->image.width;
+	image.height = uf->image.height;
+	image.depth = 6;
+	image.rowstride = image.width * image.depth;
+	image.buffer = g_new(guint8, image.height * image.rowstride);
+	// Develop complete raw image into buffer.
+	guint8 *rowbuf;
+	for (row = 0; row < image.height; row++) {
+	    rowbuf = &image.buffer[row*image.rowstride];
+	    if (row % 25 == 0)
+		preview_progress(uf->widget, _("Converting image"),
+		    0.5 * row/image.height);
+	    develope(rowbuf, rawImage[row*rowStride],
+		uf->developer, 16, (guint16 *)rowbuf, image.width);
+	}
+	// Write rotated image to output.
+	for (row = 0; row < height; row++) {
+	    if (row % 25 == 0)
+		preview_progress(uf->widget, _("Saving image"),
+		    0.5 + 0.5*row/height);
+	    ufraw_rotate_row(&image, pixbuf, uf->conf->rotationAngle,
+		bitDepth, top+row, left, width);
+	    if (grayscaleMode) {
+	      if (bitDepth > 8)
+		grayscale_16bit(graybuf, pixbuf, width);
+	      else
+		grayscale_8bit((guint8*)graybuf, (guint8*)pixbuf, width);
+	    }
+	    if (row_writer(uf, out, finalbuf, row, width, grayscaleMode) != UFRAW_SUCCESS)
+		break;
+	}
+	g_free(image.buffer);
+    } else {
+	// No rotation required. Develop straight to output.
+	for (row=0; row<height; row++) {
+	    if (row % 25 == 0)
+		preview_progress(uf->widget, _("Saving image"),
+		    0.5 + 0.5*row/height);
+	    develope(pixbuf, rawImage[(top+row)*rowStride+left],
+	        uf->developer, bitDepth, pixbuf, width);
+	    if (grayscaleMode) {
+	      if (bitDepth > 8)
+		grayscale_16bit(graybuf, pixbuf, width);
+	      else
+		grayscale_8bit((guint8*)graybuf, (guint8*)pixbuf, width);
+	    }
+	    if (row_writer(uf, out, finalbuf, row, width, grayscaleMode) != UFRAW_SUCCESS)
+		break;
+	}
+    }
+    g_free(pixbuf);
+    g_free(graybuf);
+}
+
 int ufraw_write_image(ufraw_data *uf)
 {
     /* 'volatile' supresses clobbering warning */
@@ -111,12 +284,9 @@ int ufraw_write_image(ufraw_data *uf)
 #ifdef HAVE_LIBCFITSIO
     fitsfile *fitsFile;
 #endif
-    int width, height, row, rowStride, i;
-    int left, top;
-    image_type *rawImage;
-    guint8 *pixbuf8=NULL;
-    guint16 *pixbuf16;
+    int width, height, left, top;
     char * volatile confFilename=NULL;
+    int grayscaleMode = uf->conf->grayscaleMode != grayscale_none;
     ufraw_message_reset(uf);
 
     if ( uf->conf->createID==only_id ||
@@ -135,7 +305,7 @@ int ufraw_write_image(ufraw_data *uf)
 	return status;
     }
 #ifdef HAVE_LIBTIFF
-    if (uf->conf->type==tiff8_type || uf->conf->type==tiff16_type) {
+    if ( uf->conf->type==tiff_type ) {
 	TIFFSetErrorHandler(tiff_messenger);
 	TIFFSetWarningHandler(tiff_messenger);
 	ufraw_tiff_message[0] = '\0';
@@ -200,65 +370,40 @@ int ufraw_write_image(ufraw_data *uf)
     }
     // TODO: error handling
     ufraw_convert_image(uf);
-    left = uf->conf->CropX1 * uf->image.width / uf->initialWidth;
-    top = uf->conf->CropY1 * uf->image.height / uf->initialHeight;
+    left = uf->conf->CropX1 * uf->image.width / uf->rotatedWidth;
+    top = uf->conf->CropY1 * uf->image.height / uf->rotatedHeight;
+    volatile int BitDepth = uf->conf->profile[out_profile]
+			[uf->conf->profileIndex[out_profile]].BitDepth;
+    if ( BitDepth!=16 ) BitDepth = 8;
     width = (uf->conf->CropX2 - uf->conf->CropX1)
 	    * uf->image.width / uf->initialWidth;
     height = (uf->conf->CropY2 - uf->conf->CropY1)
 	    * uf->image.height / uf->initialHeight;
-    rowStride = uf->image.width;
-    rawImage = uf->image.image;
-    pixbuf16 = g_new(guint16, width*3);
-    if (uf->conf->type==ppm8_type) {
-	fprintf(out, "P6\n%d %d\n%d\n", width, height, 0xFF);
-	pixbuf8 = g_new(guint8, width*3);
-	for (row=0; row<height; row++) {
-	    if (row%100==99)
-		preview_progress(uf->widget, _("Saving image"),
-			0.5 + 0.5*row/height);
-	    develope(pixbuf8, rawImage[(top+row)*rowStride+left],
-		    uf->developer, 8, pixbuf16, width);
-	    if ((int)fwrite(pixbuf8, 3, width, out)<width) {
-		ufraw_set_error(uf, _("Error creating file '%s'."),
-			uf->conf->outputFilename);
-		ufraw_set_error(uf, g_strerror(errno));
-		break;
-	    }
-	}
-    } else if (uf->conf->type==ppm16_type) {
-	fprintf(out, "P6\n%d %d\n%d\n", width, height, 0xFFFF);
-	for (row=0; row<height; row++) {
-	    if (row%100==99)
-		preview_progress(uf->widget, _("Saving image"),
-			0.5 + 0.5*row/height);
-	    develope(pixbuf16, rawImage[(top+row)*rowStride+left],
-		    uf->developer, 16, pixbuf16, width);
-	    for (i=0; i<3*width; i++)
-		pixbuf16[i] = g_htons(pixbuf16[i]);
-	    if ((int)fwrite(pixbuf16, 6, width, out)<width) {
-		ufraw_set_error(uf, _("Error creating file '%s'."),
-			uf->conf->outputFilename);
-		ufraw_set_error(uf, g_strerror(errno));
-		break;
-	    }
-	}
+    if ( uf->conf->type==ppm_type && BitDepth==8 ) {
+	fprintf(out, "P%c\n%d %d\n%d\n",
+		grayscaleMode ? '5' : '6', width, height, 0xFF);
+	write_image_data(uf, out, width, height, left, top,
+			 BitDepth, grayscaleMode, ppm8_row_writer);
+    } else if ( uf->conf->type==ppm_type && BitDepth==16 ) {
+	fprintf(out, "P%c\n%d %d\n%d\n",
+		grayscaleMode ? '5' : '6', width, height, 0xFFFF);
+	write_image_data(uf, out, width, height, left, top,
+			 BitDepth, grayscaleMode, ppm16_row_writer);
 #ifdef HAVE_LIBTIFF
-    } else if (uf->conf->type==tiff8_type ||
-	    uf->conf->type==tiff16_type) {
+    } else if ( uf->conf->type==tiff_type ) {
 	TIFFSetField(out, TIFFTAG_IMAGEWIDTH, width);
 	TIFFSetField(out, TIFFTAG_IMAGELENGTH, height);
 	TIFFSetField(out, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);
-	TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 3);
-	if (uf->conf->type==tiff8_type)
-	    TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, 8);
-	else
-	    TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, 16);
+	TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, grayscaleMode ? 1 : 3);
+	TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitDepth);
 	TIFFSetField(out, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
-	TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
+	TIFFSetField(out, TIFFTAG_PHOTOMETRIC, grayscaleMode
+		     ? PHOTOMETRIC_MINISBLACK : PHOTOMETRIC_RGB);
 #ifdef HAVE_LIBZ
 	if (uf->conf->losslessCompress) {
 	    TIFFSetField(out, TIFFTAG_COMPRESSION, COMPRESSION_DEFLATE);
 	    TIFFSetField(out, TIFFTAG_ZIPQUALITY, 9);
+	    TIFFSetField(out, TIFFTAG_PREDICTOR, 2);
 	}
 	else
 #endif
@@ -279,39 +424,16 @@ int ufraw_write_image(ufraw_data *uf)
 	    }
 	}
 	TIFFSetField(out, TIFFTAG_ROWSPERSTRIP, TIFFDefaultStripSize(out, 0));
-	if (uf->conf->type==tiff8_type) {
-	    pixbuf8 = g_new(guint8, width*3);
-	    for (row=0; row<height; row++) {
-		if (row%100==99) preview_progress(uf->widget, _("Saving image"),
-			    0.5+0.5*row/height);
-		develope(pixbuf8, rawImage[(top+row)*rowStride+left],
-			uf->developer, 8, pixbuf16, width);
-		if (TIFFWriteScanline(out, pixbuf8, row, 0)<0) {
-		    // 'errno' does seem to contain useful information
-		    ufraw_set_error(uf, _("Error creating file."));
-		    ufraw_set_error(uf, ufraw_tiff_message);
-		    ufraw_tiff_message[0] = '\0';
-		    break;
-		}
-	    }
-	} else {
-	    for (row=0; row<height; row++) {
-		if (row%100==99) preview_progress(uf->widget, _("Saving image"),
-			    0.5+0.5*row/height);
-		develope(pixbuf16, rawImage[(top+row)*rowStride+left],
-			uf->developer, 16, pixbuf16, width);
-		if (TIFFWriteScanline(out, pixbuf16, row, 0)<0) {
-		    // 'errno' does seem to contain useful information
-		    ufraw_set_error(uf, _("Error creating file."));
-		    ufraw_set_error(uf, ufraw_tiff_message);
-		    ufraw_tiff_message[0] = '\0';
-		    break;
-		}
-	    }
-	}
+
+	write_image_data(uf, out, width, height, left, top,
+			 BitDepth, grayscaleMode, tiff_row_writer);
+
 #endif /*HAVE_LIBTIFF*/
 #ifdef HAVE_LIBJPEG
     } else if (uf->conf->type==jpeg_type) {
+	if ( BitDepth!=8 )
+		ufraw_set_warning(uf,
+			_("Unsupported bit depth '%d' ignored."), BitDepth);
 	struct jpeg_compress_struct cinfo;
 	struct jpeg_error_mgr jerr;
 
@@ -323,10 +445,20 @@ int ufraw_write_image(ufraw_data *uf)
 	jpeg_stdio_dest(&cinfo, out);
 	cinfo.image_width = width;
 	cinfo.image_height = height;
-	cinfo.input_components = 3;
-	cinfo.in_color_space = JCS_RGB;
+	if (grayscaleMode) {
+	    cinfo.input_components = 1;
+	    cinfo.in_color_space = JCS_GRAYSCALE;
+	}
+	else {
+	    cinfo.input_components = 3;
+	    cinfo.in_color_space = JCS_RGB;
+	}
 	jpeg_set_defaults(&cinfo);
 	jpeg_set_quality(&cinfo, uf->conf->compression, TRUE);
+	if ( uf->conf->compression>90 )
+	    cinfo.comp_info[0].v_samp_factor = 1;
+	if ( uf->conf->compression>92 )
+	    cinfo.comp_info[0].h_samp_factor = 1;
 	if (uf->conf->progressiveJPEG)
 	    jpeg_simple_progression(&cinfo);
 
@@ -347,25 +479,23 @@ int ufraw_write_image(ufraw_data *uf)
 			uf->conf->outputFilename);
 	    }
 	}
-	if (uf->exifBuf!=NULL && uf->conf->embedExif) {
-	    if (uf->exifBufLen>65533) {
-		ufraw_set_warning(uf,
-			_("EXIF buffer length %d, too long, ignored."),
-			uf->exifBufLen);
-	    } else {
-		jpeg_write_marker(&cinfo, JPEG_APP0+1,
-			uf->exifBuf, uf->exifBufLen);
+	if ( uf->conf->embedExif ) {
+	    ufraw_exif_prepare_output(uf);
+	    if ( uf->outputExifBuf!=NULL ) {
+		if (uf->outputExifBufLen>65533) {
+		    ufraw_set_warning(uf,
+			    _("EXIF buffer length %d, too long, ignored."),
+			    uf->outputExifBufLen);
+		} else {
+		    jpeg_write_marker(&cinfo, JPEG_APP0+1,
+			    uf->outputExifBuf, uf->outputExifBufLen);
+		}
 	    }
 	}
-	pixbuf8 = g_new(guint8, width*3);
-	for (row=0; row<height; row++) {
-	    if (row%100==99) preview_progress(uf->widget, _("Saving image"),
-			0.5 + 0.5*row/height);
-	    develope(pixbuf8, rawImage[(top+row)*rowStride+left],
-		    uf->developer, 8, pixbuf16, width);
-	    jpeg_write_scanlines(&cinfo, &pixbuf8, 1);
-	    if ( ufraw_is_error(uf) ) break;
-	}
+
+	write_image_data(uf, &cinfo, width, height, left, top,
+			 8, grayscaleMode, jpeg_row_writer);
+
 	if ( ufraw_is_error(uf) ) {
 	    char *message = g_strdup(ufraw_get_message(uf));
 	    ufraw_message_reset(uf);
@@ -378,7 +508,7 @@ int ufraw_write_image(ufraw_data *uf)
 	jpeg_destroy_compress(&cinfo);
 #endif /*HAVE_LIBJPEG*/
 #ifdef HAVE_LIBPNG
-    } else if (uf->conf->type==png8_type || uf->conf->type==png16_type) {
+    } else if ( uf->conf->type==png_type ) {
 	png_structp png = png_create_write_struct(PNG_LIBPNG_VER_STRING,
 		uf, png_error_handler, png_warning_handler);
 	png_infop info = png_create_info_struct(png);
@@ -392,10 +522,10 @@ int ufraw_write_image(ufraw_data *uf)
 	    png_destroy_write_struct(&png, &info);
 	} else {
 	    png_init_io(png, out);
-	    int bit_depth = uf->conf->type==png8_type ? 8 : 16;
-	    png_set_IHDR(png, info, width, height, bit_depth,
-		    PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE,
-		    PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
+	    png_set_IHDR(png, info, width, height, BitDepth,
+		    grayscaleMode ? PNG_COLOR_TYPE_GRAY : PNG_COLOR_TYPE_RGB,
+		    PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE,
+		    PNG_FILTER_TYPE_BASE);
 	    png_set_compression_level(png, Z_BEST_COMPRESSION);
 	    png_text text[2];
 	    text[0].compression = PNG_TEXT_COMPRESSION_NONE;
@@ -425,30 +555,19 @@ int ufraw_write_image(ufraw_data *uf)
 			uf->conf->outputFilename);
 		}
 	    }
-	    if (uf->exifBuf!=NULL && uf->conf->embedExif)
-		PNGwriteRawProfile(png, info, "exif",
-			uf->exifBuf, uf->exifBufLen);
-	    png_write_info(png, info);
-	    if (uf->conf->type==png8_type) {
-		pixbuf8 = g_new(guint8, width*3);
-		for (row=0; row<height; row++) {
-		    if (row%100==99) preview_progress(uf->widget,
-			    _("Saving image"), 0.5+0.5*row/height);
-		    develope(pixbuf8, rawImage[(top+row)*rowStride+left],
-			    uf->developer, 8, pixbuf16, width);
-		    png_write_row(png, pixbuf8);
-		}
-	    } else {
-		if ( G_BYTE_ORDER==G_LITTLE_ENDIAN )
-		    png_set_swap(png); // Swap byte order to big-endian
-		for (row=0; row<height; row++) {
-		    if (row%100==99) preview_progress(uf->widget,
-			    _("Saving image"), 0.5+0.5*row/height);
-		    develope(pixbuf16, rawImage[(top+row)*rowStride+left],
-			    uf->developer, 16, pixbuf16, width);
-		    png_write_row(png, (guint8 *)pixbuf16);
-		}
+	    if ( uf->conf->embedExif ) {
+		ufraw_exif_prepare_output(uf);
+		if ( uf->outputExifBuf!=NULL )
+		    PNGwriteRawProfile(png, info, "exif",
+			    uf->outputExifBuf, uf->outputExifBufLen);
 	    }
+	    png_write_info(png, info);
+	    if (BitDepth != 8 && G_BYTE_ORDER==G_LITTLE_ENDIAN )
+		png_set_swap(png); // Swap byte order to big-endian
+
+	    write_image_data(uf, png, width, height, left, top,
+			     BitDepth, grayscaleMode, png_row_writer);
+
 	    png_write_end(png, NULL);
 	    png_destroy_write_struct(&png, &info);
 	}
@@ -472,6 +591,12 @@ int ufraw_write_image(ufraw_data *uf)
 
 	image = g_new(guint16, 3 * dim);
 
+	int row;
+	int i;
+	image_type *rawImage = uf->image.image;
+	int rowStride = uf->image.width;
+	guint16 pixbuf16[3];
+
 	for (row=0; row<height; row++) {
 	    if (row%100==99)
 		preview_progress(uf->widget, _("Saving image"),
@@ -569,8 +694,8 @@ int ufraw_write_image(ufraw_data *uf)
 	}
 
 	if (strlen(uf->conf->make) > 0 )
-	    fits_update_key(fitsFile, TSTRING, "MANUFRAC", &uf->conf->make,
-		    "Manufractor of the Camera", &status);
+	    fits_update_key(fitsFile, TSTRING, "MANUFACT", &uf->conf->make,
+		    "Camera Manufacturer", &status);
 
 	if (strlen(uf->conf->model) > 0 )
 	    fits_update_key(fitsFile, TSTRING, "INSTRUME", &uf->conf->model,
@@ -601,10 +726,8 @@ int ufraw_write_image(ufraw_data *uf)
 		uf->conf->outputFilename);
 	ufraw_set_error(uf, _("Unknown file type %d."), uf->conf->type);
     }
-    g_free(pixbuf16);
-    g_free(pixbuf8);
 #ifdef HAVE_LIBTIFF
-    if (uf->conf->type==tiff8_type || uf->conf->type==tiff16_type) {
+    if ( uf->conf->type==tiff_type ) {
 	TIFFClose(out);
 	if ( ufraw_tiff_message[0]!='\0' ) {
 	    if ( !ufraw_is_error(uf) ) { // Error was not already set before
@@ -612,6 +735,9 @@ int ufraw_write_image(ufraw_data *uf)
 		ufraw_set_error(uf, ufraw_tiff_message);
 	    }
 	    ufraw_tiff_message[0] = '\0';
+	} else { 
+	    if ( uf->conf->embedExif )
+		ufraw_exif_write(uf);
 	}
     } else
 #endif
@@ -631,6 +757,8 @@ int ufraw_write_image(ufraw_data *uf)
 	    }
     }
     if (uf->conf->createID==also_id) {
+	if ( ufraw_get_message(uf)!=NULL )
+	    ufraw_message(UFRAW_SET_LOG, ufraw_get_message(uf));
 	// TODO: error handling
 	conf_save(uf->conf, confFilename, NULL);
 	g_free(confFilename);
@@ -678,6 +806,7 @@ static void PNGwriteRawProfile(png_struct *ping,
 
     dp += description_length;
     *dp++='\n';
+    *dp='\0';
 
     g_snprintf(dp, allocated_length-strlen(text[0].text), "%8lu ", length);
 
